# -*- coding: utf-8 -*-
#
# File: config.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher<hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'

# EmailHeader Condition
# Vocabulary for Header fields according to rfc 2822, rfc2045, rfc2183
vo_headers = ("From",
              "Sender",
              "Reply-To",
              "To",
              "Cc",
              "Bcc",
              "Message-ID",
              "In-Reply-To",
              "References",
              "Subject",
              "Comments",
              "Keywords",
              "Resent-Date",
              "Resent-From",
              "Resent-Sender",
              "Resent-To",
              "Resent-Cc",
              "Resent-Bcc",
              "Resent-Message-ID",
              "Return-Path",
              "Received",
              "Mime-Version",
              "Content-Type",
              "Content-Transfer-Encoding",
              "Content-ID",
              "Content-Description",
              "Content-Disposition",
              )

vo_headers_default = "Subject"

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: sizeofmail.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher<hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 36831 $"
__version__   = '$Revision: 1.7 $'[11:-2]

from persistent import Persistent
from OFS.SimpleItem import SimpleItem

from zope.interface import implements, Interface
from zope.component import adapts
from zope.formlib import form
from zope import schema

from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 

from mailtoplone.base.interfaces import IEmail
from mailtoplone.contentrules import baseMessageFactory as _

class ISizeOfMailCondition(Interface):
    """Interface for the configurable aspects of a SizeOfMail condition.
    
    This is also used to create add and edit forms, below.
    """
    operator = schema.Choice(title=_(u"Relational operator"),
                                    description=_(u"The relational operator to be used"),
                                    required=True,
                                    values=['>=','<='],
                                    default='>=')
    size = schema.Int(title=_(u"Size"),
                                    description=_(u"The size to check for in megabytes"),
                                    required=True)

         
class SizeOfMailCondition(SimpleItem):
    """The actual persistent implementation of the SizeOfMail condition.
    
    Note that we must mix in Explicit to keep Zope 2 security happy.
    """
    implements(ISizeOfMailCondition, IRuleElementData)
    operator = u''
    size = u''
    element = "mailtoplone.contentrules.conditions.SizeOfMail"
    
    @property
    def summary(self):
        return _(u"Check if mail is ${operator} ${size} megabytes", mapping=dict(operator=self.operator,size=self.size))

class SizeOfMailConditionExecutor(object):
    """The executor for this condition.
    
    This is registered as an adapter in configure.zcml
    """
    implements(IExecutable)
    adapts(Interface, ISizeOfMailCondition, Interface)
         
    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        operator = self.element.operator
        size = self.element.size * 1024 * 1024
        if not IEmail.providedBy(obj):
            return False
       
        if operator == '>=':
            return len(obj.data) >= size

        if operator == '<=':
            return len(obj.data) <= size

        return False
        
class SizeOfMailAddForm(AddForm):
    """An add form for SizeOfMail rule conditions.
    """
    form_fields = form.FormFields(ISizeOfMailCondition)
    label = _(u"Add SizeOfMail Condition")
    description = _(u"An SizeOfMail Condition checks the size of a mail (<=, >=) against a user specified size in megabyte")
    form_name = _(u"Configure element")
    
    def create(self, data):
        c = SizeOfMailCondition()
        form.applyChanges(c, self.form_fields, data)
        return c

class SizeOfMailEditForm(EditForm):
    """An edit form for portal type conditions
    
    Formlib does all the magic here.
    """
    form_fields = form.FormFields(ISizeOfMailCondition)
    label = _(u"Edit SizeOfMail Condition")
    description = _(u"An SizeOfMail Condition checks the size of a mail (<=, >=) against a user specified size in megabyte")
    form_name = _(u"Configure element")

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: test_setup.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher <hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'

import unittest
from zope import component

from mailtoplone.contentrules.tests.base import MailToPloneContentrulesTestCase
from Products.CMFCore.utils import getToolByName

from zope.interface import implements, Interface
from zope.component import getUtility, getMultiAdapter

from zope.component.interfaces import IObjectEvent

from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IRuleCondition
from plone.contentrules.rule.interfaces import IRuleAction
from plone.contentrules.rule.interfaces import IExecutable

from plone.app.contentrules.rule import Rule

from mailtoplone.contentrules.conditions.emailheader import EmailHeaderCondition, EmailHeaderEditForm
from mailtoplone.contentrules.conditions.haspartoftype import HasPartOfTypeCondition, HasPartOfTypeEditForm
from mailtoplone.contentrules.conditions.sizeofmail import SizeOfMailCondition, SizeOfMailEditForm
from mailtoplone.contentrules.actions.deliver import DeliverAction, DeliverEditForm

MULTIMSG="""MIME-Version: 1.0
Content-Type: multipart/mixed;
	boundary="next"
Date: Fri, 11 Jan 2008 14:22:33 +0100
From: "Nobody"
To: "Nobody"

--next
Content-Type: text/plain;
--next
Content-Type: text/calendar;
--next
Content-Type: image/png;
--next--
"""

class DummyEvent(object):
    implements(IObjectEvent)
    
    def __init__(self, obj):
        self.object = obj


class TestSetup(MailToPloneContentrulesTestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.portal.invokeFactory('InBox', 'inbox')
    
    # condition: EmailHeader
    def testEmailHeaderRegistered(self):
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.EmailHeader')
        self.assertEquals('mailtoplone.contentrules.conditions.EmailHeader', element.addview)
        self.assertEquals('edit', element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)
    
    def testEmailHeaderInvokeAddView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.EmailHeader')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        
        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter((adding, self.portal.REQUEST), name=element.addview)
        
        addview.createAndAdd(data={'header' : 'Subject', 'value' : 'Somevalue'})
        
        e = rule.conditions[0]
        self.failUnless(isinstance(e, EmailHeaderCondition))
        self.assertEquals('Subject', e.header)
        self.assertEquals('Somevalue', e.value)
    
    def testEmailHeaderInvokeEditView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.EmailHeader')
        e = EmailHeaderCondition()
        editview = getMultiAdapter((e, self.folder.REQUEST), name=element.editview)
        self.failUnless(isinstance(editview, EmailHeaderEditForm))

    def testEmailHeaderExecutor(self):
        e = EmailHeaderCondition()
        e.header = 'Subject'
        e.value = 'test'

        self.portal.inbox.invokeFactory('Email', 'e1')
        self.portal.inbox.e1.data = 'Subject: test'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        self.portal.inbox.invokeFactory('Email', 'e2')
        self.portal.inbox.e2.data = 'Subject: laa'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e2)), IExecutable)
        self.assertEquals(False, ex())

        self.portal.inbox.invokeFactory('Email', 'e3')
        self.portal.inbox.e2.data = 'laa: test'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e3)), IExecutable)
        self.assertEquals(False, ex())

        self.portal.inbox.invokeFactory('Email', 'e4')
        self.portal.inbox.e2.data = ''
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e4)), IExecutable)
        self.assertEquals(False, ex())

        #let's test a malformed regex
        e.value = '*'

        self.portal.inbox.invokeFactory('Email', 'e5')
        self.portal.inbox.e5.data = 'Subject: BornToBeA*'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e5)), IExecutable)
        self.assertEquals(True, ex())
        
        self.portal.inbox.invokeFactory('Email', 'e6')
        self.portal.inbox.e6.data = 'Subject: BornToBeWild'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e6)), IExecutable)
        self.assertEquals(False, ex())

    # condition: HasPartOfType
    def testHasPartOfTypeRegistered(self):
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.HasPartOfType')
        self.assertEquals('mailtoplone.contentrules.conditions.HasPartOfType', element.addview)
        self.assertEquals('edit', element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)
    
    def testHasPartOfTypeInvokeAddView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.HasPartOfType')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        
        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter((adding, self.portal.REQUEST), name=element.addview)
        
        addview.createAndAdd(data={'maintype' : 'text', 'subtype' : 'plain'})
        
        e = rule.conditions[0]
        self.failUnless(isinstance(e, HasPartOfTypeCondition))
        self.assertEquals('text', e.maintype)
        self.assertEquals('plain', e.subtype)
    
    def testHasPartOfTypeInvokeEditView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.HasPartOfType')
        e = HasPartOfTypeCondition()
        editview = getMultiAdapter((e, self.folder.REQUEST), name=element.editview)
        self.failUnless(isinstance(editview, HasPartOfTypeEditForm))

    def testHasPartOfTypeExecutor(self):
        e = HasPartOfTypeCondition()
        self.portal.inbox.invokeFactory('Email', 'e1')
        self.portal.inbox.e1.data = MULTIMSG
        
        e.maintype = 'text'
        e.subtype = 'calendar'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.maintype = 'image'
        e.subtype = 'png'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.maintype = 'video'
        e.subtype = 'mpeg'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(False, ex())

        e.maintype = 'video'
        e.subtype = ''
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(False, ex())

        e.maintype = ''
        e.subtype = 'mpeg'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(False, ex())

        e.maintype = 'text'
        e.subtype = ''
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.maintype = ''
        e.subtype = 'plain'
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.maintype = ''
        e.subtype = ''
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

    # condition: SizeOfMail
    def testSizeOfMailRegistered(self):
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.SizeOfMail')
        self.assertEquals('mailtoplone.contentrules.conditions.SizeOfMail', element.addview)
        self.assertEquals('edit', element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)
    
    def testSizeOfMailInvokeAddView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.SizeOfMail')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        
        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter((adding, self.portal.REQUEST), name=element.addview)
        
        addview.createAndAdd(data={'operator' : '>=', 'size' : '10'})
        
        e = rule.conditions[0]
        self.failUnless(isinstance(e, SizeOfMailCondition))
        self.assertEquals('>=', e.operator)
        self.assertEquals('10', e.size)
    
    def testSizeOfMailInvokeEditView(self): 
        element = getUtility(IRuleCondition, name='mailtoplone.contentrules.conditions.SizeOfMail')
        e = SizeOfMailCondition()
        editview = getMultiAdapter((e, self.folder.REQUEST), name=element.editview)
        self.failUnless(isinstance(editview, SizeOfMailEditForm))

    def testSizeOfMailExecutor(self):
        e = SizeOfMailCondition()
        self.portal.inbox.invokeFactory('Email', 'e1')
        self.portal.inbox.e1.data = "x" * 1024 * 1024 * 2
        
        e.operator = '>='
        e.size = 2
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.operator = '<='
        e.size = 2
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.operator = '>='
        e.size = 1
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

        e.operator = '<='
        e.size = 1
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(False, ex())

        e.operator = '>='
        e.size = 3
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(False, ex())

        e.operator = '<='
        e.size = 3
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        self.assertEquals(True, ex())

    # action: Deliver
    def testDeliverRegistered(self):
        element = getUtility(IRuleAction, name='mailtoplone.contentrules.actions.deliver')
        self.assertEquals('mailtoplone.contentrules.actions.deliver', element.addview)
        self.assertEquals('edit', element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)
    
    def testDeliverInvokeAddView(self): 
        element = getUtility(IRuleAction, name='mailtoplone.contentrules.actions.deliver')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        
        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        addview = getMultiAdapter((adding, self.portal.REQUEST), name=element.addview)
        
        addview.createAndAdd(data={'key' : 'searchkey'})
        
        e = rule.actions[0]
        self.failUnless(isinstance(e, DeliverAction))
        self.assertEquals('searchkey', e.key)
    
    def testDeliverInvokeEditView(self): 
        element = getUtility(IRuleAction, name='mailtoplone.contentrules.actions.deliver')
        e = DeliverAction()
        editview = getMultiAdapter((e, self.folder.REQUEST), name=element.editview)
        self.failUnless(isinstance(editview, DeliverEditForm))

    def testDeliverExecutor(self):
        self.portal.invokeFactory('InBox','destinbox1')
        self.portal.invokeFactory('InBox','destinbox2')
        e = DeliverAction()
        e.key = 'destinbox1'

        self.portal.inbox.invokeFactory('Email', 'e1')
        ex = getMultiAdapter((self.portal, e, DummyEvent(self.portal.inbox.e1)), IExecutable)
        ex()

        self.assertEquals(True, len(self.portal.destinbox1.objectIds()) == 1)
        self.assertEquals(True, len(self.portal.destinbox2.objectIds()) == 0)
        self.assertEquals(True, len(self.portal.inbox.objectIds()) == 1)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetup))
    return suite

# vim: set ft=python ts=4 sw=4 expandtab :




# -*- coding: utf-8 -*-
#
# File: haspartoftype.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher<hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 36831 $"
__version__   = '$Revision: 1.7 $'[11:-2]

from persistent import Persistent
from OFS.SimpleItem import SimpleItem

from zope.interface import implements, Interface
from zope.component import adapts
from zope.formlib import form
from zope import schema

from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 

import email
from mailtoplone.base.interfaces import IEmail
from mailtoplone.contentrules import baseMessageFactory as _

class IHasPartOfTypeCondition(Interface):
    """Interface for the configurable aspects of a HasPartOfType condition.
    
    This is also used to create add and edit forms, below.
    """
    maintype = schema.TextLine(title=_(u"Maintype"),
                                    description=_(u"The maintype to check for"),
                                    required=False)
    subtype = schema.TextLine(title=_(u"Subtype"),
                                    description=_(u"The subtype to check for"),
                                    required=False)
         
class HasPartOfTypeCondition(SimpleItem):
    """The actual persistent implementation of the HasPartOfType condition.
    
    Note that we must mix in Explicit to keep Zope 2 security happy.
    """
    implements(IHasPartOfTypeCondition, IRuleElementData)
    maintype = u''
    subtype = u''
    element = "mailtoplone.contentrules.conditions.HasPartOfType"
    
    @property
    def summary(self):
        return _(u"Check if the email contains a part with maintype/subtype pair ${maintype}/${subtype}", mapping=dict(maintype=self.maintype,subtype=self.subtype))

class HasPartOfTypeConditionExecutor(object):
    """The executor for this condition.
    
    This is registered as an adapter in configure.zcml
    """
    implements(IExecutable)
    adapts(Interface, IHasPartOfTypeCondition, Interface)
         
    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        maintype = self.element.maintype
        subtype = self.element.subtype
        if not IEmail.providedBy(obj):
            return False
        
        mailobj = email.message_from_string(obj.data)
        for part in mailobj.walk():
            if maintype and subtype:
                if part.get_content_maintype() == maintype and part.get_content_subtype() == subtype:
                    return True
            if maintype and not subtype:
                if part.get_content_maintype() == maintype:
                    return True
            if not maintype and subtype:
                if part.get_content_subtype() == subtype:
                    return True
            if not maintype and not subtype:
                return True
        return False

class HasPartOfTypeAddForm(AddForm):
    """An add form for HasPartOfType rule conditions.
    """
    form_fields = form.FormFields(IHasPartOfTypeCondition)
    label = _(u"Add HasPartOfType Condition")
    description = _(u"A HasPartOfType Condition checks if the email contains a part of specified maintype/subtype")
    form_name = _(u"Configure element")
    
    def create(self, data):
        c = HasPartOfTypeCondition()
        form.applyChanges(c, self.form_fields, data)
        return c

class HasPartOfTypeEditForm(EditForm):
    """An edit form for portal type conditions
    
    Formlib does all the magic here.
    """
    form_fields = form.FormFields(IHasPartOfTypeCondition)
    label = _(u"Edit HasPartOfType Condition")
    description = _(u"A HasPartOfType Condition checks if the email contains a part of specified maintype/subtype")
    form_name = _(u"Configure element")

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: deliver.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher<hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 36831 $"
__version__   = '$Revision: 1.7 $'[11:-2]

from OFS.SimpleItem import SimpleItem
from persistent import Persistent 

from zope.interface import implements, Interface
from zope.component import adapts, getUtility
from zope.formlib import form
from zope import schema

import OFS.subscribers

from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 
import transaction

from Acquisition import aq_inner, aq_parent, aq_base
from ZODB.POSException import ConflictError
from Products.CMFCore.utils import getToolByName

from Products.CMFPlone import utils

from mailtoplone.base.interfaces import IMailDropBoxFactory, IMailDropBox
from mailtoplone.contentrules import baseMessageFactory as _

class IDeliverAction(Interface):
    """Interface for the configurable aspects of a deliver action.
    
    This is also used to create add and edit forms, below.
    """
    
    key = schema.TextLine(title=_(u"Key"),
                                  description=_(u"Key used to determine the DropBox"),
                                  required=True,
                                  )

class DeliverAction(SimpleItem):
    """The actual persistent implementation of the action element.
    """
    implements(IDeliverAction, IRuleElementData)
    
    key = ''
    element = 'mailtoplone.contentrules.actions.deliver'
    
    @property
    def summary(self):
        return _(u"Deliver to DropBoxes by key ${key}", mapping=dict(key=self.key))
    
class DeliverActionExecutor(object):
    """The executor for this action.
    """
    implements(IExecutable)
    adapts(Interface, IDeliverAction, Interface)
         
    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        
        obj = self.event.object
        parent = aq_parent(aq_inner(obj))
        key = self.element.key
        dropbox_factory = getUtility(IMailDropBoxFactory)
        dropboxes = dropbox_factory(obj, key)

        for dropbox in dropboxes:
            if not dropbox is parent:
                IMailDropBox(dropbox).drop(obj.data)

        return True
        
class DeliverAddForm(AddForm):
    """An add form for deliver actions.
    """
    form_fields = form.FormFields(IDeliverAction)
    label = _(u"Add Deliver Action")
    description = _(u"A deliver action can drop an email to (a) different inbox(es).")
    form_name = _(u"Configure element")
    
    def create(self, data):
        a = DeliverAction()
        form.applyChanges(a, self.form_fields, data)
        return a

class DeliverEditForm(EditForm):
    """An edit form for deliver rule actions.
    
    Formlib does all the magic here.
    """
    form_fields = form.FormFields(IDeliverAction)
    label = _(u"Edit Deliver Action")
    description = _(u"A deliver action can drop an email to (a) different inbox(es).")
    form_name = _(u"Configure element")

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: emailheader.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Hans-Peter Locher<hans-peter.locher@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 36831 $"
__version__   = '$Revision: 1.7 $'[11:-2]

from persistent import Persistent
from OFS.SimpleItem import SimpleItem

from zope.interface import implements, Interface
from zope.component import adapts
from zope.formlib import form
from zope import schema

from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 

import email
import re
from mailtoplone.base.interfaces import IEmail
from mailtoplone.contentrules import baseMessageFactory as _
from mailtoplone.contentrules.config import vo_headers, vo_headers_default

class IEmailHeaderCondition(Interface):
    """Interface for the configurable aspects of a EmailHeader condition.
    
    This is also used to create add and edit forms, below.
    """
    header = schema.Choice(title=_(u"Email Header"),
                                    description=_(u"The Email Header to check for"),
                                    required=True,
                                    values=vo_headers,
                                    default=vo_headers_default)
    value = schema.TextLine(title=_(u"Value"),
                                    description=_(u"The Value to check for"),
                                    required=True)

         
class EmailHeaderCondition(SimpleItem):
    """The actual persistent implementation of the file extension condition.
    
    Note that we must mix in Explicit to keep Zope 2 security happy.
    """
    implements(IEmailHeaderCondition, IRuleElementData)
    header = u''
    value = u''
    element = "mailtoplone.contentrules.conditions.EmailHeader"
    
    @property
    def summary(self):
        return _(u"Check ${header} for ${value}", mapping=dict(header=self.header,value=self.value))

class EmailHeaderConditionExecutor(object):
    """The executor for this condition.
    
    This is registered as an adapter in configure.zcml
    """
    implements(IExecutable)
    adapts(Interface, IEmailHeaderCondition, Interface)
         
    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        # TODO: 
        # - recursive functionality:add checkbox (go recursively through mail?)
        obj = self.event.object
        header = self.element.header
        value = self.element.value
        
        if not IEmail.providedBy(obj):
            return False
        
        mailobj = email.message_from_string(obj.data)
        if not header in mailobj.keys():
            return False
        
        # The user may enter a malformed regex, in that case, the input should be interpreted as
        # a string
        try:
            exp = re.compile(value)
        except:
            return value in mailobj.get(header)

        if re.search(exp,mailobj.get(header)):
            return True

        return False
        
class EmailHeaderAddForm(AddForm):
    """An add form for EmailHeader rule conditions.
    """
    form_fields = form.FormFields(IEmailHeaderCondition)
    label = _(u"Add EmailHeader Condition")
    description = _(u"An EmailHeader Condition checks a specified header for a specified value, specify value as regular expression")
    form_name = _(u"Configure element")
    
    def create(self, data):
        c = EmailHeaderCondition()
        form.applyChanges(c, self.form_fields, data)
        return c

class EmailHeaderEditForm(EditForm):
    """An edit form for portal type conditions
    
    Formlib does all the magic here.
    """
    form_fields = form.FormFields(IEmailHeaderCondition)
    label = _(u"Edit EmailHeader Condition")
    description = _(u"An EmailHeader Condition checks a specified header for a specified value, value can be a regular expression")
    form_name = _(u"Configure element")

# vim: set ft=python ts=4 sw=4 expandtab :
